import CRUD from '../index';
import { RestUrl } from '../types';
/**
 * 创建一个crud单实例入口并返回
 * @param {string | RestUrl} restURL url字符串或包含url属性的配置对象
 * @return {CRUD} $crud
 */
export declare function useCrud(restURL: string | RestUrl): CRUD;
/**
 * 创建一个crud多实例入口并返回
 * @param {Record<string,string | RestUrl>} restURL 明确指定实例key的url配置信息
 * @return {Record<string,CRUD>} $cruds
 */
export declare function useCruds(restURL: Record<string, string | RestUrl>): Record<string, CRUD>;
/**
 * 用于注册钩子
 * @param {string} hookName 钩子名称
 * @param {Function} hook 回调函数
 */
export declare function onHook(hookName: string, hook: (crud: CRUD, ...args: any[]) => void): void;
/**
 * 用于自定义组件向上查找最近的$crud并返回。用于自定义组件封装crud逻辑
 * @param {string} [crudName] 如果使用了cruds就必须指定crudName
 * @return {CRUD | null} $crud 向上查找最近的crud实例或null
 */
export declare function lookUpCrud(crudName?: string): CRUD | null;
export default CRUD;
