import CRUD from '../index';
import { RestUrl } from '../types';
/**
 * 创建一个crud单实例入口并绑定指定的vm(需要crud的vue组件)
 * @param {object} vm 需要使用crud的组件实例
 * @param {string | RestUrl} restURL url字符串或包含url属性的配置对象
 * @return {CRUD} $crud
 */
export declare function useCrud(vm: Record<string, any>, restURL: string | RestUrl): CRUD;
/**
 * 创建crud多实例入口并绑定指定的vm(需要crud的vue组件)
 * @param {object} vm 需要使用crud的组件实例
 * @param {Record<string, string | RestUrl>} restURLMap restURL配置对象
 * @return {Record<string, CRUD>} $cruds
 */
export declare function useCruds(vm: Record<string, any>, restURLMap: Record<string, string | RestUrl>): Record<string, CRUD>;
/**
 * 用于自定义组件向上查找最近的$crud并返回。用于自定义组件封装crud逻辑
 *
 * @param {object} vm 自定义组件实例
 * @param {string} [crudName] 如果使用了cruds就必须指定crudName
 * @return {CRUD | null} $crud 向上查找最近的crud实例或null
 */
export declare function lookUpCrud(vm: Record<string, any>, crudName?: string): CRUD | null;
export default CRUD;
