import { RestUrl, Pagination } from './types';
/**
 * CRUD容器。每个CRUD服务对应一个实例，提供查询、表格、分页等逻辑托管
 */
declare class CRUD {
    static request: Function;
    static install: (app: any, options: any) => void;
    static HOOK: {
        BEFORE_QUERY: string;
        AFTER_QUERY: string;
        BEFORE_DELETE: string;
        AFTER_DELETE: string;
        BEFORE_ADD: string;
        BEFORE_EDIT_QUERY: string;
        BEFORE_EDIT: string;
        BEFORE_VIEW_QUERY: string;
        BEFORE_VIEW: string;
        BEFORE_SUBMIT: string;
        AFTER_SUBMIT: string;
        BEFORE_EXPORT: string;
        AFTER_EXPORT: string;
        ON_ERROR: string;
        ON_CANCEL: string;
        BEFORE_IMPORT: string;
        AFTER_IMPORT: string;
        BEFORE_SORT: string;
        BEFORE_SUBMIT_SORT: string;
        AFTER_SUBMIT_SORT: string;
    };
    static RESTAPI: {
        QUERY: string;
        EXPORT: string;
        IMPORT: string;
        SORT: string;
        ADD: string;
        UPDATE: string;
        DELETE: string;
    };
    static defaults: {
        query: Record<string, string>;
        view: Record<string, boolean | undefined>;
        pagination: Pagination;
        table: Record<string, string>;
        [k: string]: Function | Record<string, string> | Record<string, boolean | undefined> | Pagination;
    };
    params: Record<string, any>;
    vm: Record<string, unknown>;
    private url;
    private urlVar;
    view: Record<string, boolean | undefined>;
    loading: {
        query: boolean;
        table: boolean;
        del: boolean;
        export: boolean;
        form: boolean;
        submit: boolean;
        import: boolean;
        sort: boolean;
    };
    query: {};
    table: {
        rowKey: string;
        data: Record<string, unknown>[];
        selection: Record<string, unknown>[];
        allColumns: Record<string, unknown>[];
        orders: Record<string, unknown>[];
    };
    pagination: {
        _pageSize: number;
        pageSize: number;
        currentPage: number;
        total: number;
    };
    sortation: {};
    formStatus: number;
    form: {};
    error: {
        name: string;
        message: string;
        status: string;
    };
    constructor(restURL: string | RestUrl);
    setURLParams(v: Record<string, unknown>): void;
    changeSelection(selection: Record<string, any>[]): void;
    changeOrder(orderData: {
        column: Record<string, any>;
        prop: string;
        order: string | null;
    }): void;
    getRestURL(): string;
    toQuery(): Promise<unknown>;
    toDelete(rows: Record<string, unknown> | Record<string, unknown>[]): Promise<unknown>;
    toExport(): Promise<unknown>;
    toImport(file: File | File[]): Promise<unknown>;
    toSort(...args: unknown[]): void;
    submitSort(): Promise<unknown>;
    toAdd(...args: unknown[]): void;
    toEdit(row: Record<string, unknown>): Promise<unknown>;
    toView(row: Record<string, unknown>): Promise<unknown>;
    cancel(): void;
    submit(_formEl: unknown): Promise<unknown>;
    reload(): Promise<unknown>;
    getDetails(id: string, params: Record<string, unknown>): Promise<unknown>;
    private doQuery;
    private doAdd;
    private doUpdate;
    private doDelete;
    private doExport;
    private doImport;
    private doSort;
}
export declare function crudWarn(...args: unknown[]): void;
export declare function crudError(...args: unknown[]): void;
export default CRUD;
